# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import status_retrieve_all_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.status_response import StatusResponse
from ..types.status_retrieve_all_response import StatusRetrieveAllResponse

__all__ = ["StatusResource", "AsyncStatusResource"]


class StatusResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> StatusResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/legalesign/legalesign-rest-python#accessing-raw-response-data-eg-headers
        """
        return StatusResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StatusResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/legalesign/legalesign-rest-python#with_streaming_response
        """
        return StatusResourceWithStreamingResponse(self)

    def retrieve(
        self,
        doc_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatusResponse:
        """
        Faster short query for a document status

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not doc_id:
            raise ValueError(f"Expected a non-empty value for `doc_id` but received {doc_id!r}")
        return self._get(
            f"/status/{doc_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatusResponse,
        )

    def retrieve_all(
        self,
        *,
        filter: str | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatusRetrieveAllResponse:
        """
        Shortened faster query for status of signing documents

        Args:
          filter: Filter on archived status, default is false

          limit: Length of dataset to return. Use with offset query to iterate through results.

          offset: Offset from start of dataset. Use with the limit query to iterate through
              dataset.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/status/",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "limit": limit,
                        "offset": offset,
                    },
                    status_retrieve_all_params.StatusRetrieveAllParams,
                ),
            ),
            cast_to=StatusRetrieveAllResponse,
        )


class AsyncStatusResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncStatusResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/legalesign/legalesign-rest-python#accessing-raw-response-data-eg-headers
        """
        return AsyncStatusResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStatusResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/legalesign/legalesign-rest-python#with_streaming_response
        """
        return AsyncStatusResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        doc_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatusResponse:
        """
        Faster short query for a document status

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not doc_id:
            raise ValueError(f"Expected a non-empty value for `doc_id` but received {doc_id!r}")
        return await self._get(
            f"/status/{doc_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=StatusResponse,
        )

    async def retrieve_all(
        self,
        *,
        filter: str | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> StatusRetrieveAllResponse:
        """
        Shortened faster query for status of signing documents

        Args:
          filter: Filter on archived status, default is false

          limit: Length of dataset to return. Use with offset query to iterate through results.

          offset: Offset from start of dataset. Use with the limit query to iterate through
              dataset.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/status/",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "limit": limit,
                        "offset": offset,
                    },
                    status_retrieve_all_params.StatusRetrieveAllParams,
                ),
            ),
            cast_to=StatusRetrieveAllResponse,
        )


class StatusResourceWithRawResponse:
    def __init__(self, status: StatusResource) -> None:
        self._status = status

        self.retrieve = to_raw_response_wrapper(
            status.retrieve,
        )
        self.retrieve_all = to_raw_response_wrapper(
            status.retrieve_all,
        )


class AsyncStatusResourceWithRawResponse:
    def __init__(self, status: AsyncStatusResource) -> None:
        self._status = status

        self.retrieve = async_to_raw_response_wrapper(
            status.retrieve,
        )
        self.retrieve_all = async_to_raw_response_wrapper(
            status.retrieve_all,
        )


class StatusResourceWithStreamingResponse:
    def __init__(self, status: StatusResource) -> None:
        self._status = status

        self.retrieve = to_streamed_response_wrapper(
            status.retrieve,
        )
        self.retrieve_all = to_streamed_response_wrapper(
            status.retrieve_all,
        )


class AsyncStatusResourceWithStreamingResponse:
    def __init__(self, status: AsyncStatusResource) -> None:
        self._status = status

        self.retrieve = async_to_streamed_response_wrapper(
            status.retrieve,
        )
        self.retrieve_all = async_to_streamed_response_wrapper(
            status.retrieve_all,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .list_meta import ListMeta
from .member_response import MemberResponse

__all__ = ["MemberListResponse"]


class MemberListResponse(BaseModel):
    meta: Optional[ListMeta] = None

    objects: Optional[List[MemberResponse]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["GroupRetrieveResponse"]


class GroupRetrieveResponse(BaseModel):
    created: Optional[datetime] = None

    default_email: Optional[str] = None

    default_extraemail: Optional[str] = None

    footer: Optional[str] = None
    """html of content"""

    footer_height: Optional[int] = None

    header: Optional[str] = None

    is_active: Optional[bool] = None

    members: Optional[List[str]] = None
    """list of members uris"""

    modified: Optional[datetime] = None

    name: Optional[str] = None

    pagesize: Optional[int] = None

    public_name: Optional[str] = None

    resource_uri: Optional[str] = None

    slug: Optional[str] = None

    user: Optional[str] = None

    xframe_allow: Optional[bool] = None

    xframe_allow_pdf_edit: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import httpx
import pytest
from respx import MockRouter

from legalesign import Legalesign, AsyncLegalesign
from tests.utils import assert_matches_type
from legalesign.types import (
    DocumentListResponse,
    DocumentCreateResponse,
    DocumentRetrieveResponse,
    DocumentGetFieldsResponse,
)
from legalesign._utils import parse_datetime
from legalesign._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDocument:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Legalesign) -> None:
        document = client.document.create(
            group="https://example.com",
            name="x",
            signers=[
                {
                    "email": "dev@stainless.com",
                    "firstname": "firstname",
                    "lastname": "lastname",
                }
            ],
        )
        assert_matches_type(DocumentCreateResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Legalesign) -> None:
        document = client.document.create(
            group="https://example.com",
            name="x",
            signers=[
                {
                    "email": "dev@stainless.com",
                    "firstname": "firstname",
                    "lastname": "lastname",
                    "attachments": ["/api/v1/attachment/IK-GV--w1tvt/"],
                    "behalfof": "behalfof",
                    "decide_later": True,
                    "expires": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "message": "message",
                    "order": 0,
                    "reviewers": [
                        {
                            "email": "dev@stainless.com",
                            "firstname": "firstname",
                            "include_link": True,
                            "lastname": "lastname",
                        }
                    ],
                    "role": "witness",
                    "sms": "sms",
                    "subject": "subject",
                    "timezone": "timezone",
                }
            ],
            append_pdf=True,
            auto_archive=True,
            cc_emails="cc_emails",
            convert_sender_to_signer=True,
            do_email=True,
            footer="footer",
            footer_height=0,
            header="header",
            header_height=0,
            pdf_password="pdf_password",
            pdf_password_type=1,
            pdftext={"foo": "string"},
            redirect="https://",
            reminders="",
            return_signer_links=True,
            signature_type=0,
            signers_in_order=True,
            signertext={"foo": "string"},
            strict_fields=True,
            tag="tag",
            tag1="tag1",
            tag2="tag2",
            template="https://example.com",
            templatepdf="https://example.com",
            text="text",
            user="https://example.com",
        )
        assert_matches_type(DocumentCreateResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Legalesign) -> None:
        response = client.document.with_raw_response.create(
            group="https://example.com",
            name="x",
            signers=[
                {
                    "email": "dev@stainless.com",
                    "firstname": "firstname",
                    "lastname": "lastname",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(DocumentCreateResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Legalesign) -> None:
        with client.document.with_streaming_response.create(
            group="https://example.com",
            name="x",
            signers=[
                {
                    "email": "dev@stainless.com",
                    "firstname": "firstname",
                    "lastname": "lastname",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(DocumentCreateResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Legalesign) -> None:
        document = client.document.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DocumentRetrieveResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Legalesign) -> None:
        response = client.document.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(DocumentRetrieveResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Legalesign) -> None:
        with client.document.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(DocumentRetrieveResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Legalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            client.document.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Legalesign) -> None:
        document = client.document.list(
            group="group",
        )
        assert_matches_type(DocumentListResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Legalesign) -> None:
        document = client.document.list(
            group="group",
            archived="archived",
            created_gt=parse_datetime("2019-12-27T18:11:19.117Z"),
            email="email",
            limit=0,
            modified_gt=parse_datetime("2019-12-27T18:11:19.117Z"),
            nosigners="nosigners",
            offset=0,
            status=0,
        )
        assert_matches_type(DocumentListResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Legalesign) -> None:
        response = client.document.with_raw_response.list(
            group="group",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(DocumentListResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Legalesign) -> None:
        with client.document.with_streaming_response.list(
            group="group",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(DocumentListResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_archive(self, client: Legalesign) -> None:
        document = client.document.archive(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_archive(self, client: Legalesign) -> None:
        response = client.document.with_raw_response.archive(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_archive(self, client: Legalesign) -> None:
        with client.document.with_streaming_response.archive(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert document is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_archive(self, client: Legalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            client.document.with_raw_response.archive(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete_permanently(self, client: Legalesign) -> None:
        document = client.document.delete_permanently(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete_permanently(self, client: Legalesign) -> None:
        response = client.document.with_raw_response.delete_permanently(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete_permanently(self, client: Legalesign) -> None:
        with client.document.with_streaming_response.delete_permanently(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert document is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete_permanently(self, client: Legalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            client.document.with_raw_response.delete_permanently(
                "",
            )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_download_audit_log(self, client: Legalesign, respx_mock: MockRouter) -> None:
        respx_mock.get("/document/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/auditlog/").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        document = client.document.download_audit_log(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert document.is_closed
        assert document.json() == {"foo": "bar"}
        assert cast(Any, document.is_closed) is True
        assert isinstance(document, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_download_audit_log(self, client: Legalesign, respx_mock: MockRouter) -> None:
        respx_mock.get("/document/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/auditlog/").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        document = client.document.with_raw_response.download_audit_log(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert document.is_closed is True
        assert document.http_request.headers.get("X-Stainless-Lang") == "python"
        assert document.json() == {"foo": "bar"}
        assert isinstance(document, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_download_audit_log(self, client: Legalesign, respx_mock: MockRouter) -> None:
        respx_mock.get("/document/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/auditlog/").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        with client.document.with_streaming_response.download_audit_log(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as document:
            assert not document.is_closed
            assert document.http_request.headers.get("X-Stainless-Lang") == "python"

            assert document.json() == {"foo": "bar"}
            assert cast(Any, document.is_closed) is True
            assert isinstance(document, StreamedBinaryAPIResponse)

        assert cast(Any, document.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_path_params_download_audit_log(self, client: Legalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            client.document.with_raw_response.download_audit_log(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_fields(self, client: Legalesign) -> None:
        document = client.document.get_fields(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DocumentGetFieldsResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_fields(self, client: Legalesign) -> None:
        response = client.document.with_raw_response.get_fields(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert_matches_type(DocumentGetFieldsResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_fields(self, client: Legalesign) -> None:
        with client.document.with_streaming_response.get_fields(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert_matches_type(DocumentGetFieldsResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get_fields(self, client: Legalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            client.document.with_raw_response.get_fields(
                "",
            )

    @pytest.mark.skip(reason="Prism doesn't properly handle redirects")
    @parametrize
    def test_method_preview(self, client: Legalesign) -> None:
        document = client.document.preview()
        assert document is None

    @pytest.mark.skip(reason="Prism doesn't properly handle redirects")
    @parametrize
    def test_method_preview_with_all_params(self, client: Legalesign) -> None:
        document = client.document.preview(
            group="/api/v1/group/IK-GV--w1tvt/",
            signee_count=0,
            text="text",
            title="title",
        )
        assert document is None

    @pytest.mark.skip(reason="Prism doesn't properly handle redirects")
    @parametrize
    def test_raw_response_preview(self, client: Legalesign) -> None:
        response = client.document.with_raw_response.preview()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = response.parse()
        assert document is None

    @pytest.mark.skip(reason="Prism doesn't properly handle redirects")
    @parametrize
    def test_streaming_response_preview(self, client: Legalesign) -> None:
        with client.document.with_streaming_response.preview() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = response.parse()
            assert document is None

        assert cast(Any, response.is_closed) is True


class TestAsyncDocument:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncLegalesign) -> None:
        document = await async_client.document.create(
            group="https://example.com",
            name="x",
            signers=[
                {
                    "email": "dev@stainless.com",
                    "firstname": "firstname",
                    "lastname": "lastname",
                }
            ],
        )
        assert_matches_type(DocumentCreateResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncLegalesign) -> None:
        document = await async_client.document.create(
            group="https://example.com",
            name="x",
            signers=[
                {
                    "email": "dev@stainless.com",
                    "firstname": "firstname",
                    "lastname": "lastname",
                    "attachments": ["/api/v1/attachment/IK-GV--w1tvt/"],
                    "behalfof": "behalfof",
                    "decide_later": True,
                    "expires": parse_datetime("2019-12-27T18:11:19.117Z"),
                    "message": "message",
                    "order": 0,
                    "reviewers": [
                        {
                            "email": "dev@stainless.com",
                            "firstname": "firstname",
                            "include_link": True,
                            "lastname": "lastname",
                        }
                    ],
                    "role": "witness",
                    "sms": "sms",
                    "subject": "subject",
                    "timezone": "timezone",
                }
            ],
            append_pdf=True,
            auto_archive=True,
            cc_emails="cc_emails",
            convert_sender_to_signer=True,
            do_email=True,
            footer="footer",
            footer_height=0,
            header="header",
            header_height=0,
            pdf_password="pdf_password",
            pdf_password_type=1,
            pdftext={"foo": "string"},
            redirect="https://",
            reminders="",
            return_signer_links=True,
            signature_type=0,
            signers_in_order=True,
            signertext={"foo": "string"},
            strict_fields=True,
            tag="tag",
            tag1="tag1",
            tag2="tag2",
            template="https://example.com",
            templatepdf="https://example.com",
            text="text",
            user="https://example.com",
        )
        assert_matches_type(DocumentCreateResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncLegalesign) -> None:
        response = await async_client.document.with_raw_response.create(
            group="https://example.com",
            name="x",
            signers=[
                {
                    "email": "dev@stainless.com",
                    "firstname": "firstname",
                    "lastname": "lastname",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(DocumentCreateResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncLegalesign) -> None:
        async with async_client.document.with_streaming_response.create(
            group="https://example.com",
            name="x",
            signers=[
                {
                    "email": "dev@stainless.com",
                    "firstname": "firstname",
                    "lastname": "lastname",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(DocumentCreateResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncLegalesign) -> None:
        document = await async_client.document.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DocumentRetrieveResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncLegalesign) -> None:
        response = await async_client.document.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(DocumentRetrieveResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncLegalesign) -> None:
        async with async_client.document.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(DocumentRetrieveResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncLegalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            await async_client.document.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncLegalesign) -> None:
        document = await async_client.document.list(
            group="group",
        )
        assert_matches_type(DocumentListResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncLegalesign) -> None:
        document = await async_client.document.list(
            group="group",
            archived="archived",
            created_gt=parse_datetime("2019-12-27T18:11:19.117Z"),
            email="email",
            limit=0,
            modified_gt=parse_datetime("2019-12-27T18:11:19.117Z"),
            nosigners="nosigners",
            offset=0,
            status=0,
        )
        assert_matches_type(DocumentListResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncLegalesign) -> None:
        response = await async_client.document.with_raw_response.list(
            group="group",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(DocumentListResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncLegalesign) -> None:
        async with async_client.document.with_streaming_response.list(
            group="group",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(DocumentListResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_archive(self, async_client: AsyncLegalesign) -> None:
        document = await async_client.document.archive(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_archive(self, async_client: AsyncLegalesign) -> None:
        response = await async_client.document.with_raw_response.archive(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_archive(self, async_client: AsyncLegalesign) -> None:
        async with async_client.document.with_streaming_response.archive(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert document is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_archive(self, async_client: AsyncLegalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            await async_client.document.with_raw_response.archive(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete_permanently(self, async_client: AsyncLegalesign) -> None:
        document = await async_client.document.delete_permanently(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete_permanently(self, async_client: AsyncLegalesign) -> None:
        response = await async_client.document.with_raw_response.delete_permanently(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert document is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete_permanently(self, async_client: AsyncLegalesign) -> None:
        async with async_client.document.with_streaming_response.delete_permanently(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert document is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete_permanently(self, async_client: AsyncLegalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            await async_client.document.with_raw_response.delete_permanently(
                "",
            )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_download_audit_log(self, async_client: AsyncLegalesign, respx_mock: MockRouter) -> None:
        respx_mock.get("/document/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/auditlog/").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        document = await async_client.document.download_audit_log(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert document.is_closed
        assert await document.json() == {"foo": "bar"}
        assert cast(Any, document.is_closed) is True
        assert isinstance(document, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_download_audit_log(self, async_client: AsyncLegalesign, respx_mock: MockRouter) -> None:
        respx_mock.get("/document/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/auditlog/").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )

        document = await async_client.document.with_raw_response.download_audit_log(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert document.is_closed is True
        assert document.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await document.json() == {"foo": "bar"}
        assert isinstance(document, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_download_audit_log(
        self, async_client: AsyncLegalesign, respx_mock: MockRouter
    ) -> None:
        respx_mock.get("/document/182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e/auditlog/").mock(
            return_value=httpx.Response(200, json={"foo": "bar"})
        )
        async with async_client.document.with_streaming_response.download_audit_log(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as document:
            assert not document.is_closed
            assert document.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await document.json() == {"foo": "bar"}
            assert cast(Any, document.is_closed) is True
            assert isinstance(document, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, document.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_path_params_download_audit_log(self, async_client: AsyncLegalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            await async_client.document.with_raw_response.download_audit_log(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_fields(self, async_client: AsyncLegalesign) -> None:
        document = await async_client.document.get_fields(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DocumentGetFieldsResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_fields(self, async_client: AsyncLegalesign) -> None:
        response = await async_client.document.with_raw_response.get_fields(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert_matches_type(DocumentGetFieldsResponse, document, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_fields(self, async_client: AsyncLegalesign) -> None:
        async with async_client.document.with_streaming_response.get_fields(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert_matches_type(DocumentGetFieldsResponse, document, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get_fields(self, async_client: AsyncLegalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            await async_client.document.with_raw_response.get_fields(
                "",
            )

    @pytest.mark.skip(reason="Prism doesn't properly handle redirects")
    @parametrize
    async def test_method_preview(self, async_client: AsyncLegalesign) -> None:
        document = await async_client.document.preview()
        assert document is None

    @pytest.mark.skip(reason="Prism doesn't properly handle redirects")
    @parametrize
    async def test_method_preview_with_all_params(self, async_client: AsyncLegalesign) -> None:
        document = await async_client.document.preview(
            group="/api/v1/group/IK-GV--w1tvt/",
            signee_count=0,
            text="text",
            title="title",
        )
        assert document is None

    @pytest.mark.skip(reason="Prism doesn't properly handle redirects")
    @parametrize
    async def test_raw_response_preview(self, async_client: AsyncLegalesign) -> None:
        response = await async_client.document.with_raw_response.preview()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        document = await response.parse()
        assert document is None

    @pytest.mark.skip(reason="Prism doesn't properly handle redirects")
    @parametrize
    async def test_streaming_response_preview(self, async_client: AsyncLegalesign) -> None:
        async with async_client.document.with_streaming_response.preview() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            document = await response.parse()
            assert document is None

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from legalesign import Legalesign, AsyncLegalesign
from tests.utils import assert_matches_type
from legalesign.types.templatepdf import FieldListResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestFields:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Legalesign) -> None:
        field = client.templatepdf.fields.create(
            pdf_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            body=[
                {
                    "ax": 0,
                    "ay": 0,
                    "bx": 0,
                    "by": 0,
                    "element_type": "signature",
                    "page": 0,
                    "signer": 1,
                }
            ],
        )
        assert field is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Legalesign) -> None:
        response = client.templatepdf.fields.with_raw_response.create(
            pdf_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            body=[
                {
                    "ax": 0,
                    "ay": 0,
                    "bx": 0,
                    "by": 0,
                    "element_type": "signature",
                    "page": 0,
                    "signer": 1,
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        field = response.parse()
        assert field is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Legalesign) -> None:
        with client.templatepdf.fields.with_streaming_response.create(
            pdf_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            body=[
                {
                    "ax": 0,
                    "ay": 0,
                    "bx": 0,
                    "by": 0,
                    "element_type": "signature",
                    "page": 0,
                    "signer": 1,
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            field = response.parse()
            assert field is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Legalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pdf_id` but received ''"):
            client.templatepdf.fields.with_raw_response.create(
                pdf_id="",
                body=[
                    {
                        "ax": 0,
                        "ay": 0,
                        "bx": 0,
                        "by": 0,
                        "element_type": "signature",
                        "page": 0,
                        "signer": 1,
                    }
                ],
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Legalesign) -> None:
        field = client.templatepdf.fields.list(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(FieldListResponse, field, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Legalesign) -> None:
        response = client.templatepdf.fields.with_raw_response.list(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        field = response.parse()
        assert_matches_type(FieldListResponse, field, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Legalesign) -> None:
        with client.templatepdf.fields.with_streaming_response.list(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            field = response.parse()
            assert_matches_type(FieldListResponse, field, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Legalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pdf_id` but received ''"):
            client.templatepdf.fields.with_raw_response.list(
                "",
            )


class TestAsyncFields:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncLegalesign) -> None:
        field = await async_client.templatepdf.fields.create(
            pdf_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            body=[
                {
                    "ax": 0,
                    "ay": 0,
                    "bx": 0,
                    "by": 0,
                    "element_type": "signature",
                    "page": 0,
                    "signer": 1,
                }
            ],
        )
        assert field is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncLegalesign) -> None:
        response = await async_client.templatepdf.fields.with_raw_response.create(
            pdf_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            body=[
                {
                    "ax": 0,
                    "ay": 0,
                    "bx": 0,
                    "by": 0,
                    "element_type": "signature",
                    "page": 0,
                    "signer": 1,
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        field = await response.parse()
        assert field is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncLegalesign) -> None:
        async with async_client.templatepdf.fields.with_streaming_response.create(
            pdf_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            body=[
                {
                    "ax": 0,
                    "ay": 0,
                    "bx": 0,
                    "by": 0,
                    "element_type": "signature",
                    "page": 0,
                    "signer": 1,
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            field = await response.parse()
            assert field is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncLegalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pdf_id` but received ''"):
            await async_client.templatepdf.fields.with_raw_response.create(
                pdf_id="",
                body=[
                    {
                        "ax": 0,
                        "ay": 0,
                        "bx": 0,
                        "by": 0,
                        "element_type": "signature",
                        "page": 0,
                        "signer": 1,
                    }
                ],
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncLegalesign) -> None:
        field = await async_client.templatepdf.fields.list(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(FieldListResponse, field, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncLegalesign) -> None:
        response = await async_client.templatepdf.fields.with_raw_response.list(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        field = await response.parse()
        assert_matches_type(FieldListResponse, field, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncLegalesign) -> None:
        async with async_client.templatepdf.fields.with_streaming_response.list(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            field = await response.parse()
            assert_matches_type(FieldListResponse, field, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncLegalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pdf_id` but received ''"):
            await async_client.templatepdf.fields.with_raw_response.list(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .fields import (
    FieldsResource,
    AsyncFieldsResource,
    FieldsResourceWithRawResponse,
    AsyncFieldsResourceWithRawResponse,
    FieldsResourceWithStreamingResponse,
    AsyncFieldsResourceWithStreamingResponse,
)
from .templatepdf import (
    TemplatepdfResource,
    AsyncTemplatepdfResource,
    TemplatepdfResourceWithRawResponse,
    AsyncTemplatepdfResourceWithRawResponse,
    TemplatepdfResourceWithStreamingResponse,
    AsyncTemplatepdfResourceWithStreamingResponse,
)

__all__ = [
    "FieldsResource",
    "AsyncFieldsResource",
    "FieldsResourceWithRawResponse",
    "AsyncFieldsResourceWithRawResponse",
    "FieldsResourceWithStreamingResponse",
    "AsyncFieldsResourceWithStreamingResponse",
    "TemplatepdfResource",
    "AsyncTemplatepdfResource",
    "TemplatepdfResourceWithRawResponse",
    "AsyncTemplatepdfResourceWithRawResponse",
    "TemplatepdfResourceWithStreamingResponse",
    "AsyncTemplatepdfResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["InvitedListParams"]


class InvitedListParams(TypedDict, total=False):
    group: str
    """filter list by a given group"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._version import __version__
from .resources import (
    pdf,
    user,
    group,
    member,
    signer,
    status,
    invited,
    document,
    template,
    subscribe,
    attachment,
    unsubscribe,
    notifications,
)
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError, LegalesignError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)
from .resources.templatepdf import templatepdf

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Legalesign",
    "AsyncLegalesign",
    "Client",
    "AsyncClient",
]


class Legalesign(SyncAPIClient):
    attachment: attachment.AttachmentResource
    document: document.DocumentResource
    group: group.GroupResource
    invited: invited.InvitedResource
    member: member.MemberResource
    notifications: notifications.NotificationsResource
    pdf: pdf.PdfResource
    signer: signer.SignerResource
    status: status.StatusResource
    subscribe: subscribe.SubscribeResource
    template: template.TemplateResource
    templatepdf: templatepdf.TemplatepdfResource
    unsubscribe: unsubscribe.UnsubscribeResource
    user: user.UserResource
    with_raw_response: LegalesignWithRawResponse
    with_streaming_response: LegalesignWithStreamedResponse

    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Legalesign client instance.

        This automatically infers the `api_key` argument from the `LEGALESIGN_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("LEGALESIGN_API_KEY")
        if api_key is None:
            raise LegalesignError(
                "The api_key client option must be set either by passing api_key to the client or by setting the LEGALESIGN_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("LEGALESIGN_BASE_URL")
        if base_url is None:
            base_url = f"https://lon-dev.legalesign.com/api/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.attachment = attachment.AttachmentResource(self)
        self.document = document.DocumentResource(self)
        self.group = group.GroupResource(self)
        self.invited = invited.InvitedResource(self)
        self.member = member.MemberResource(self)
        self.notifications = notifications.NotificationsResource(self)
        self.pdf = pdf.PdfResource(self)
        self.signer = signer.SignerResource(self)
        self.status = status.StatusResource(self)
        self.subscribe = subscribe.SubscribeResource(self)
        self.template = template.TemplateResource(self)
        self.templatepdf = templatepdf.TemplatepdfResource(self)
        self.unsubscribe = unsubscribe.UnsubscribeResource(self)
        self.user = user.UserResource(self)
        self.with_raw_response = LegalesignWithRawResponse(self)
        self.with_streaming_response = LegalesignWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncLegalesign(AsyncAPIClient):
    attachment: attachment.AsyncAttachmentResource
    document: document.AsyncDocumentResource
    group: group.AsyncGroupResource
    invited: invited.AsyncInvitedResource
    member: member.AsyncMemberResource
    notifications: notifications.AsyncNotificationsResource
    pdf: pdf.AsyncPdfResource
    signer: signer.AsyncSignerResource
    status: status.AsyncStatusResource
    subscribe: subscribe.AsyncSubscribeResource
    template: template.AsyncTemplateResource
    templatepdf: templatepdf.AsyncTemplatepdfResource
    unsubscribe: unsubscribe.AsyncUnsubscribeResource
    user: user.AsyncUserResource
    with_raw_response: AsyncLegalesignWithRawResponse
    with_streaming_response: AsyncLegalesignWithStreamedResponse

    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncLegalesign client instance.

        This automatically infers the `api_key` argument from the `LEGALESIGN_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("LEGALESIGN_API_KEY")
        if api_key is None:
            raise LegalesignError(
                "The api_key client option must be set either by passing api_key to the client or by setting the LEGALESIGN_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("LEGALESIGN_BASE_URL")
        if base_url is None:
            base_url = f"https://lon-dev.legalesign.com/api/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self.attachment = attachment.AsyncAttachmentResource(self)
        self.document = document.AsyncDocumentResource(self)
        self.group = group.AsyncGroupResource(self)
        self.invited = invited.AsyncInvitedResource(self)
        self.member = member.AsyncMemberResource(self)
        self.notifications = notifications.AsyncNotificationsResource(self)
        self.pdf = pdf.AsyncPdfResource(self)
        self.signer = signer.AsyncSignerResource(self)
        self.status = status.AsyncStatusResource(self)
        self.subscribe = subscribe.AsyncSubscribeResource(self)
        self.template = template.AsyncTemplateResource(self)
        self.templatepdf = templatepdf.AsyncTemplatepdfResource(self)
        self.unsubscribe = unsubscribe.AsyncUnsubscribeResource(self)
        self.user = user.AsyncUserResource(self)
        self.with_raw_response = AsyncLegalesignWithRawResponse(self)
        self.with_streaming_response = AsyncLegalesignWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": api_key}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class LegalesignWithRawResponse:
    def __init__(self, client: Legalesign) -> None:
        self.attachment = attachment.AttachmentResourceWithRawResponse(client.attachment)
        self.document = document.DocumentResourceWithRawResponse(client.document)
        self.group = group.GroupResourceWithRawResponse(client.group)
        self.invited = invited.InvitedResourceWithRawResponse(client.invited)
        self.member = member.MemberResourceWithRawResponse(client.member)
        self.notifications = notifications.NotificationsResourceWithRawResponse(client.notifications)
        self.pdf = pdf.PdfResourceWithRawResponse(client.pdf)
        self.signer = signer.SignerResourceWithRawResponse(client.signer)
        self.status = status.StatusResourceWithRawResponse(client.status)
        self.subscribe = subscribe.SubscribeResourceWithRawResponse(client.subscribe)
        self.template = template.TemplateResourceWithRawResponse(client.template)
        self.templatepdf = templatepdf.TemplatepdfResourceWithRawResponse(client.templatepdf)
        self.unsubscribe = unsubscribe.UnsubscribeResourceWithRawResponse(client.unsubscribe)
        self.user = user.UserResourceWithRawResponse(client.user)


class AsyncLegalesignWithRawResponse:
    def __init__(self, client: AsyncLegalesign) -> None:
        self.attachment = attachment.AsyncAttachmentResourceWithRawResponse(client.attachment)
        self.document = document.AsyncDocumentResourceWithRawResponse(client.document)
        self.group = group.AsyncGroupResourceWithRawResponse(client.group)
        self.invited = invited.AsyncInvitedResourceWithRawResponse(client.invited)
        self.member = member.AsyncMemberResourceWithRawResponse(client.member)
        self.notifications = notifications.AsyncNotificationsResourceWithRawResponse(client.notifications)
        self.pdf = pdf.AsyncPdfResourceWithRawResponse(client.pdf)
        self.signer = signer.AsyncSignerResourceWithRawResponse(client.signer)
        self.status = status.AsyncStatusResourceWithRawResponse(client.status)
        self.subscribe = subscribe.AsyncSubscribeResourceWithRawResponse(client.subscribe)
        self.template = template.AsyncTemplateResourceWithRawResponse(client.template)
        self.templatepdf = templatepdf.AsyncTemplatepdfResourceWithRawResponse(client.templatepdf)
        self.unsubscribe = unsubscribe.AsyncUnsubscribeResourceWithRawResponse(client.unsubscribe)
        self.user = user.AsyncUserResourceWithRawResponse(client.user)


class LegalesignWithStreamedResponse:
    def __init__(self, client: Legalesign) -> None:
        self.attachment = attachment.AttachmentResourceWithStreamingResponse(client.attachment)
        self.document = document.DocumentResourceWithStreamingResponse(client.document)
        self.group = group.GroupResourceWithStreamingResponse(client.group)
        self.invited = invited.InvitedResourceWithStreamingResponse(client.invited)
        self.member = member.MemberResourceWithStreamingResponse(client.member)
        self.notifications = notifications.NotificationsResourceWithStreamingResponse(client.notifications)
        self.pdf = pdf.PdfResourceWithStreamingResponse(client.pdf)
        self.signer = signer.SignerResourceWithStreamingResponse(client.signer)
        self.status = status.StatusResourceWithStreamingResponse(client.status)
        self.subscribe = subscribe.SubscribeResourceWithStreamingResponse(client.subscribe)
        self.template = template.TemplateResourceWithStreamingResponse(client.template)
        self.templatepdf = templatepdf.TemplatepdfResourceWithStreamingResponse(client.templatepdf)
        self.unsubscribe = unsubscribe.UnsubscribeResourceWithStreamingResponse(client.unsubscribe)
        self.user = user.UserResourceWithStreamingResponse(client.user)


class AsyncLegalesignWithStreamedResponse:
    def __init__(self, client: AsyncLegalesign) -> None:
        self.attachment = attachment.AsyncAttachmentResourceWithStreamingResponse(client.attachment)
        self.document = document.AsyncDocumentResourceWithStreamingResponse(client.document)
        self.group = group.AsyncGroupResourceWithStreamingResponse(client.group)
        self.invited = invited.AsyncInvitedResourceWithStreamingResponse(client.invited)
        self.member = member.AsyncMemberResourceWithStreamingResponse(client.member)
        self.notifications = notifications.AsyncNotificationsResourceWithStreamingResponse(client.notifications)
        self.pdf = pdf.AsyncPdfResourceWithStreamingResponse(client.pdf)
        self.signer = signer.AsyncSignerResourceWithStreamingResponse(client.signer)
        self.status = status.AsyncStatusResourceWithStreamingResponse(client.status)
        self.subscribe = subscribe.AsyncSubscribeResourceWithStreamingResponse(client.subscribe)
        self.template = template.AsyncTemplateResourceWithStreamingResponse(client.template)
        self.templatepdf = templatepdf.AsyncTemplatepdfResourceWithStreamingResponse(client.templatepdf)
        self.unsubscribe = unsubscribe.AsyncUnsubscribeResourceWithStreamingResponse(client.unsubscribe)
        self.user = user.AsyncUserResourceWithStreamingResponse(client.user)


Client = Legalesign

AsyncClient = AsyncLegalesign



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .signer_status_enum import SignerStatusEnum

__all__ = ["StatusResponse"]


class StatusResponse(BaseModel):
    archived: Optional[bool] = None

    download_final: Optional[bool] = None

    resource_uri: Optional[str] = None

    status: Optional[SignerStatusEnum] = None
    """Signer status options:

    - 4 - unsent
    - 5 - scheduled to be sent
    - 10 - sent
    - 15 - email opened
    - 20 - visited
    - 30 - fields complete
    - 35 - fields complete ex signature
    - 39 - waiting for witness to complete
    - 40 - signed
    - 50 - downloaded
    - 60 - rejected
    """

    tag: Optional[str] = None

    tag1: Optional[str] = None

    tag2: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .._models import BaseModel
from .timezone_enum import TimezoneEnum

__all__ = ["UserRetrieveResponse"]


class UserRetrieveResponse(BaseModel):
    date_joined: Optional[datetime] = None

    email: Optional[str] = None

    first_name: Optional[str] = None

    groups: Optional[List[str]] = None

    last_login: Optional[datetime] = None

    last_name: Optional[str] = None

    resource_uri: Optional[str] = None

    timezone: Optional[TimezoneEnum] = None
    """List of available timezones"""

    username: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from legalesign import Legalesign, AsyncLegalesign

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSubscribe:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_webhook(self, client: Legalesign) -> None:
        subscribe = client.subscribe.create_webhook(
            notify="realtime",
            url="https://",
        )
        assert subscribe is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_webhook_with_all_params(self, client: Legalesign) -> None:
        subscribe = client.subscribe.create_webhook(
            notify="realtime",
            url="https://",
            event_filter="",
            group="group",
        )
        assert subscribe is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create_webhook(self, client: Legalesign) -> None:
        response = client.subscribe.with_raw_response.create_webhook(
            notify="realtime",
            url="https://",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscribe = response.parse()
        assert subscribe is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create_webhook(self, client: Legalesign) -> None:
        with client.subscribe.with_streaming_response.create_webhook(
            notify="realtime",
            url="https://",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscribe = response.parse()
            assert subscribe is None

        assert cast(Any, response.is_closed) is True


class TestAsyncSubscribe:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_webhook(self, async_client: AsyncLegalesign) -> None:
        subscribe = await async_client.subscribe.create_webhook(
            notify="realtime",
            url="https://",
        )
        assert subscribe is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_webhook_with_all_params(self, async_client: AsyncLegalesign) -> None:
        subscribe = await async_client.subscribe.create_webhook(
            notify="realtime",
            url="https://",
            event_filter="",
            group="group",
        )
        assert subscribe is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create_webhook(self, async_client: AsyncLegalesign) -> None:
        response = await async_client.subscribe.with_raw_response.create_webhook(
            notify="realtime",
            url="https://",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        subscribe = await response.parse()
        assert subscribe is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create_webhook(self, async_client: AsyncLegalesign) -> None:
        async with async_client.subscribe.with_streaming_response.create_webhook(
            notify="realtime",
            url="https://",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            subscribe = await response.parse()
            assert subscribe is None

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from legalesign import Legalesign, AsyncLegalesign
from tests.utils import assert_matches_type
from legalesign.types import (
    TemplatePdf,
    TemplatepdfListResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTemplatepdf:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Legalesign) -> None:
        templatepdf = client.templatepdf.create(
            group="/api/v1/group/IK-GV--w1tvt/",
            pdf_file="U3RhaW5sZXNzIHJvY2tz",
        )
        assert templatepdf is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Legalesign) -> None:
        templatepdf = client.templatepdf.create(
            group="/api/v1/group/IK-GV--w1tvt/",
            pdf_file="U3RhaW5sZXNzIHJvY2tz",
            archive_upon_send=True,
            process_tags=True,
            title="title",
            user="user",
        )
        assert templatepdf is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Legalesign) -> None:
        response = client.templatepdf.with_raw_response.create(
            group="/api/v1/group/IK-GV--w1tvt/",
            pdf_file="U3RhaW5sZXNzIHJvY2tz",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        templatepdf = response.parse()
        assert templatepdf is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Legalesign) -> None:
        with client.templatepdf.with_streaming_response.create(
            group="/api/v1/group/IK-GV--w1tvt/",
            pdf_file="U3RhaW5sZXNzIHJvY2tz",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            templatepdf = response.parse()
            assert templatepdf is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Legalesign) -> None:
        templatepdf = client.templatepdf.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(TemplatePdf, templatepdf, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Legalesign) -> None:
        response = client.templatepdf.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        templatepdf = response.parse()
        assert_matches_type(TemplatePdf, templatepdf, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Legalesign) -> None:
        with client.templatepdf.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            templatepdf = response.parse()
            assert_matches_type(TemplatePdf, templatepdf, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Legalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pdf_id` but received ''"):
            client.templatepdf.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Legalesign) -> None:
        templatepdf = client.templatepdf.list()
        assert_matches_type(TemplatepdfListResponse, templatepdf, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Legalesign) -> None:
        templatepdf = client.templatepdf.list(
            archive="archive",
            group="group",
            limit=0,
            offset=0,
        )
        assert_matches_type(TemplatepdfListResponse, templatepdf, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Legalesign) -> None:
        response = client.templatepdf.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        templatepdf = response.parse()
        assert_matches_type(TemplatepdfListResponse, templatepdf, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Legalesign) -> None:
        with client.templatepdf.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            templatepdf = response.parse()
            assert_matches_type(TemplatepdfListResponse, templatepdf, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_archive(self, client: Legalesign) -> None:
        templatepdf = client.templatepdf.archive(
            "pdfId",
        )
        assert templatepdf is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_archive(self, client: Legalesign) -> None:
        response = client.templatepdf.with_raw_response.archive(
            "pdfId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        templatepdf = response.parse()
        assert templatepdf is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_archive(self, client: Legalesign) -> None:
        with client.templatepdf.with_streaming_response.archive(
            "pdfId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            templatepdf = response.parse()
            assert templatepdf is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_archive(self, client: Legalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pdf_id` but received ''"):
            client.templatepdf.with_raw_response.archive(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_convert_tags(self, client: Legalesign) -> None:
        templatepdf = client.templatepdf.convert_tags(
            "pdfId",
        )
        assert templatepdf is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_convert_tags(self, client: Legalesign) -> None:
        response = client.templatepdf.with_raw_response.convert_tags(
            "pdfId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        templatepdf = response.parse()
        assert templatepdf is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_convert_tags(self, client: Legalesign) -> None:
        with client.templatepdf.with_streaming_response.convert_tags(
            "pdfId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            templatepdf = response.parse()
            assert templatepdf is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_convert_tags(self, client: Legalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pdf_id` but received ''"):
            client.templatepdf.with_raw_response.convert_tags(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_edit_link(self, client: Legalesign) -> None:
        templatepdf = client.templatepdf.get_edit_link(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(str, templatepdf, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_edit_link(self, client: Legalesign) -> None:
        response = client.templatepdf.with_raw_response.get_edit_link(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        templatepdf = response.parse()
        assert_matches_type(str, templatepdf, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_edit_link(self, client: Legalesign) -> None:
        with client.templatepdf.with_streaming_response.get_edit_link(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            templatepdf = response.parse()
            assert_matches_type(str, templatepdf, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_get_edit_link(self, client: Legalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pdf_id` but received ''"):
            client.templatepdf.with_raw_response.get_edit_link(
                "",
            )


class TestAsyncTemplatepdf:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncLegalesign) -> None:
        templatepdf = await async_client.templatepdf.create(
            group="/api/v1/group/IK-GV--w1tvt/",
            pdf_file="U3RhaW5sZXNzIHJvY2tz",
        )
        assert templatepdf is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncLegalesign) -> None:
        templatepdf = await async_client.templatepdf.create(
            group="/api/v1/group/IK-GV--w1tvt/",
            pdf_file="U3RhaW5sZXNzIHJvY2tz",
            archive_upon_send=True,
            process_tags=True,
            title="title",
            user="user",
        )
        assert templatepdf is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncLegalesign) -> None:
        response = await async_client.templatepdf.with_raw_response.create(
            group="/api/v1/group/IK-GV--w1tvt/",
            pdf_file="U3RhaW5sZXNzIHJvY2tz",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        templatepdf = await response.parse()
        assert templatepdf is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncLegalesign) -> None:
        async with async_client.templatepdf.with_streaming_response.create(
            group="/api/v1/group/IK-GV--w1tvt/",
            pdf_file="U3RhaW5sZXNzIHJvY2tz",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            templatepdf = await response.parse()
            assert templatepdf is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncLegalesign) -> None:
        templatepdf = await async_client.templatepdf.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(TemplatePdf, templatepdf, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncLegalesign) -> None:
        response = await async_client.templatepdf.with_raw_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        templatepdf = await response.parse()
        assert_matches_type(TemplatePdf, templatepdf, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncLegalesign) -> None:
        async with async_client.templatepdf.with_streaming_response.retrieve(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            templatepdf = await response.parse()
            assert_matches_type(TemplatePdf, templatepdf, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncLegalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pdf_id` but received ''"):
            await async_client.templatepdf.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncLegalesign) -> None:
        templatepdf = await async_client.templatepdf.list()
        assert_matches_type(TemplatepdfListResponse, templatepdf, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncLegalesign) -> None:
        templatepdf = await async_client.templatepdf.list(
            archive="archive",
            group="group",
            limit=0,
            offset=0,
        )
        assert_matches_type(TemplatepdfListResponse, templatepdf, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncLegalesign) -> None:
        response = await async_client.templatepdf.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        templatepdf = await response.parse()
        assert_matches_type(TemplatepdfListResponse, templatepdf, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncLegalesign) -> None:
        async with async_client.templatepdf.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            templatepdf = await response.parse()
            assert_matches_type(TemplatepdfListResponse, templatepdf, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_archive(self, async_client: AsyncLegalesign) -> None:
        templatepdf = await async_client.templatepdf.archive(
            "pdfId",
        )
        assert templatepdf is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_archive(self, async_client: AsyncLegalesign) -> None:
        response = await async_client.templatepdf.with_raw_response.archive(
            "pdfId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        templatepdf = await response.parse()
        assert templatepdf is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_archive(self, async_client: AsyncLegalesign) -> None:
        async with async_client.templatepdf.with_streaming_response.archive(
            "pdfId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            templatepdf = await response.parse()
            assert templatepdf is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_archive(self, async_client: AsyncLegalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pdf_id` but received ''"):
            await async_client.templatepdf.with_raw_response.archive(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_convert_tags(self, async_client: AsyncLegalesign) -> None:
        templatepdf = await async_client.templatepdf.convert_tags(
            "pdfId",
        )
        assert templatepdf is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_convert_tags(self, async_client: AsyncLegalesign) -> None:
        response = await async_client.templatepdf.with_raw_response.convert_tags(
            "pdfId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        templatepdf = await response.parse()
        assert templatepdf is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_convert_tags(self, async_client: AsyncLegalesign) -> None:
        async with async_client.templatepdf.with_streaming_response.convert_tags(
            "pdfId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            templatepdf = await response.parse()
            assert templatepdf is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_convert_tags(self, async_client: AsyncLegalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pdf_id` but received ''"):
            await async_client.templatepdf.with_raw_response.convert_tags(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_edit_link(self, async_client: AsyncLegalesign) -> None:
        templatepdf = await async_client.templatepdf.get_edit_link(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(str, templatepdf, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_edit_link(self, async_client: AsyncLegalesign) -> None:
        response = await async_client.templatepdf.with_raw_response.get_edit_link(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        templatepdf = await response.parse()
        assert_matches_type(str, templatepdf, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_edit_link(self, async_client: AsyncLegalesign) -> None:
        async with async_client.templatepdf.with_streaming_response.get_edit_link(
            "182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            templatepdf = await response.parse()
            assert_matches_type(str, templatepdf, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_get_edit_link(self, async_client: AsyncLegalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `pdf_id` but received ''"):
            await async_client.templatepdf.with_raw_response.get_edit_link(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["TemplatePdf"]


class TemplatePdf(BaseModel):
    created: Optional[datetime] = None

    group: Optional[str] = None

    modified: Optional[datetime] = None

    page_count: Optional[int] = None

    parties: Optional[str] = None
    """JSON stringified array of document parties"""

    resource_uri: Optional[str] = None

    signer_count: Optional[int] = None

    title: Optional[str] = None

    user: Optional[str] = None
    """resource_uri for user"""

    uuid: Optional[str] = None
    """id for pdf object"""

    valid: Optional[bool] = None
    """Is able to be sent (if fields do not validate)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import httpx
import pytest
from respx import MockRouter

from legalesign import Legalesign, AsyncLegalesign
from legalesign._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPdf:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_retrieve(self, client: Legalesign, respx_mock: MockRouter) -> None:
        respx_mock.get("/pdf/docId/").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        pdf = client.pdf.retrieve(
            "docId",
        )
        assert pdf.is_closed
        assert pdf.json() == {"foo": "bar"}
        assert cast(Any, pdf.is_closed) is True
        assert isinstance(pdf, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_retrieve(self, client: Legalesign, respx_mock: MockRouter) -> None:
        respx_mock.get("/pdf/docId/").mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        pdf = client.pdf.with_raw_response.retrieve(
            "docId",
        )

        assert pdf.is_closed is True
        assert pdf.http_request.headers.get("X-Stainless-Lang") == "python"
        assert pdf.json() == {"foo": "bar"}
        assert isinstance(pdf, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_retrieve(self, client: Legalesign, respx_mock: MockRouter) -> None:
        respx_mock.get("/pdf/docId/").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        with client.pdf.with_streaming_response.retrieve(
            "docId",
        ) as pdf:
            assert not pdf.is_closed
            assert pdf.http_request.headers.get("X-Stainless-Lang") == "python"

            assert pdf.json() == {"foo": "bar"}
            assert cast(Any, pdf.is_closed) is True
            assert isinstance(pdf, StreamedBinaryAPIResponse)

        assert cast(Any, pdf.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_path_params_retrieve(self, client: Legalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            client.pdf.with_raw_response.retrieve(
                "",
            )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_create_preview(self, client: Legalesign, respx_mock: MockRouter) -> None:
        respx_mock.post("/pdf/preview/").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        pdf = client.pdf.create_preview(
            group="/api/v1/group/IK-GV--w1tvt/",
            is_signature_per_page=0,
            signature_type=0,
            signee_count=0,
            text="text",
        )
        assert pdf.is_closed
        assert pdf.json() == {"foo": "bar"}
        assert cast(Any, pdf.is_closed) is True
        assert isinstance(pdf, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_method_create_preview_with_all_params(self, client: Legalesign, respx_mock: MockRouter) -> None:
        respx_mock.post("/pdf/preview/").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        pdf = client.pdf.create_preview(
            group="/api/v1/group/IK-GV--w1tvt/",
            is_signature_per_page=0,
            signature_type=0,
            signee_count=0,
            text="text",
            footer="footer",
            footer_height=0,
            header="header",
            header_height=0,
            pdfheader=True,
            title="title",
        )
        assert pdf.is_closed
        assert pdf.json() == {"foo": "bar"}
        assert cast(Any, pdf.is_closed) is True
        assert isinstance(pdf, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_raw_response_create_preview(self, client: Legalesign, respx_mock: MockRouter) -> None:
        respx_mock.post("/pdf/preview/").mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        pdf = client.pdf.with_raw_response.create_preview(
            group="/api/v1/group/IK-GV--w1tvt/",
            is_signature_per_page=0,
            signature_type=0,
            signee_count=0,
            text="text",
        )

        assert pdf.is_closed is True
        assert pdf.http_request.headers.get("X-Stainless-Lang") == "python"
        assert pdf.json() == {"foo": "bar"}
        assert isinstance(pdf, BinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    def test_streaming_response_create_preview(self, client: Legalesign, respx_mock: MockRouter) -> None:
        respx_mock.post("/pdf/preview/").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        with client.pdf.with_streaming_response.create_preview(
            group="/api/v1/group/IK-GV--w1tvt/",
            is_signature_per_page=0,
            signature_type=0,
            signee_count=0,
            text="text",
        ) as pdf:
            assert not pdf.is_closed
            assert pdf.http_request.headers.get("X-Stainless-Lang") == "python"

            assert pdf.json() == {"foo": "bar"}
            assert cast(Any, pdf.is_closed) is True
            assert isinstance(pdf, StreamedBinaryAPIResponse)

        assert cast(Any, pdf.is_closed) is True


class TestAsyncPdf:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_retrieve(self, async_client: AsyncLegalesign, respx_mock: MockRouter) -> None:
        respx_mock.get("/pdf/docId/").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        pdf = await async_client.pdf.retrieve(
            "docId",
        )
        assert pdf.is_closed
        assert await pdf.json() == {"foo": "bar"}
        assert cast(Any, pdf.is_closed) is True
        assert isinstance(pdf, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_retrieve(self, async_client: AsyncLegalesign, respx_mock: MockRouter) -> None:
        respx_mock.get("/pdf/docId/").mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        pdf = await async_client.pdf.with_raw_response.retrieve(
            "docId",
        )

        assert pdf.is_closed is True
        assert pdf.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await pdf.json() == {"foo": "bar"}
        assert isinstance(pdf, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_retrieve(self, async_client: AsyncLegalesign, respx_mock: MockRouter) -> None:
        respx_mock.get("/pdf/docId/").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        async with async_client.pdf.with_streaming_response.retrieve(
            "docId",
        ) as pdf:
            assert not pdf.is_closed
            assert pdf.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await pdf.json() == {"foo": "bar"}
            assert cast(Any, pdf.is_closed) is True
            assert isinstance(pdf, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, pdf.is_closed) is True

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_path_params_retrieve(self, async_client: AsyncLegalesign) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `doc_id` but received ''"):
            await async_client.pdf.with_raw_response.retrieve(
                "",
            )

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_create_preview(self, async_client: AsyncLegalesign, respx_mock: MockRouter) -> None:
        respx_mock.post("/pdf/preview/").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        pdf = await async_client.pdf.create_preview(
            group="/api/v1/group/IK-GV--w1tvt/",
            is_signature_per_page=0,
            signature_type=0,
            signee_count=0,
            text="text",
        )
        assert pdf.is_closed
        assert await pdf.json() == {"foo": "bar"}
        assert cast(Any, pdf.is_closed) is True
        assert isinstance(pdf, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_method_create_preview_with_all_params(
        self, async_client: AsyncLegalesign, respx_mock: MockRouter
    ) -> None:
        respx_mock.post("/pdf/preview/").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        pdf = await async_client.pdf.create_preview(
            group="/api/v1/group/IK-GV--w1tvt/",
            is_signature_per_page=0,
            signature_type=0,
            signee_count=0,
            text="text",
            footer="footer",
            footer_height=0,
            header="header",
            header_height=0,
            pdfheader=True,
            title="title",
        )
        assert pdf.is_closed
        assert await pdf.json() == {"foo": "bar"}
        assert cast(Any, pdf.is_closed) is True
        assert isinstance(pdf, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_raw_response_create_preview(self, async_client: AsyncLegalesign, respx_mock: MockRouter) -> None:
        respx_mock.post("/pdf/preview/").mock(return_value=httpx.Response(200, json={"foo": "bar"}))

        pdf = await async_client.pdf.with_raw_response.create_preview(
            group="/api/v1/group/IK-GV--w1tvt/",
            is_signature_per_page=0,
            signature_type=0,
            signee_count=0,
            text="text",
        )

        assert pdf.is_closed is True
        assert pdf.http_request.headers.get("X-Stainless-Lang") == "python"
        assert await pdf.json() == {"foo": "bar"}
        assert isinstance(pdf, AsyncBinaryAPIResponse)

    @parametrize
    @pytest.mark.respx(base_url=base_url)
    async def test_streaming_response_create_preview(
        self, async_client: AsyncLegalesign, respx_mock: MockRouter
    ) -> None:
        respx_mock.post("/pdf/preview/").mock(return_value=httpx.Response(200, json={"foo": "bar"}))
        async with async_client.pdf.with_streaming_response.create_preview(
            group="/api/v1/group/IK-GV--w1tvt/",
            is_signature_per_page=0,
            signature_type=0,
            signee_count=0,
            text="text",
        ) as pdf:
            assert not pdf.is_closed
            assert pdf.http_request.headers.get("X-Stainless-Lang") == "python"

            assert await pdf.json() == {"foo": "bar"}
            assert cast(Any, pdf.is_closed) is True
            assert isinstance(pdf, AsyncStreamedBinaryAPIResponse)

        assert cast(Any, pdf.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["TemplateCreateParams"]


class TemplateCreateParams(TypedDict, total=False):
    group: Required[str]

    latest_text: Required[str]
    """text/html for template"""

    title: Required[str]

    user: str
    """assign to a user if not api user"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["DocumentPreviewParams"]


class DocumentPreviewParams(TypedDict, total=False):
    group: str

    signee_count: int

    text: str

    title: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, Annotated, TypedDict

from .._types import Base64FileInput
from .._utils import PropertyInfo

__all__ = ["AttachmentUploadParams"]


class AttachmentUploadParams(TypedDict, total=False):
    filename: Required[str]
    """Simple alphanumeric name ending .pdf"""

    group: Required[str]
    """URI of the group name"""

    pdf_file: Required[Annotated[Union[str, Base64FileInput], PropertyInfo(format="base64")]]
    """Base64 encoded PDF file data, max size is a group setting, 5MB by default"""

    description: str

    user: str
    """Assign to group member if not the api user"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .permissions_enum import PermissionsEnum

__all__ = ["MemberCreateParams"]


class MemberCreateParams(TypedDict, total=False):
    email: Required[str]

    group: Required[str]

    do_email: bool
    """use legalesign to send email notification to new user"""

    permission: PermissionsEnum
    """Permissions options:

    - 1 - administrator
    - 2 - team docs visible, create & send
    - 3 - team docs visible, send only
    - 4 - no team sent docs visible, send only
    - 5 - no team docs visible, create & send
    - 6 - team docs visible, read only
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import PermissionsEnum, member_list_params, member_create_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.member_response import MemberResponse
from ..types.permissions_enum import PermissionsEnum
from ..types.member_list_response import MemberListResponse

__all__ = ["MemberResource", "AsyncMemberResource"]


class MemberResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MemberResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/legalesign/legalesign-rest-python#accessing-raw-response-data-eg-headers
        """
        return MemberResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MemberResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/legalesign/legalesign-rest-python#with_streaming_response
        """
        return MemberResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        email: str,
        group: str,
        do_email: bool | Omit = omit,
        permission: PermissionsEnum | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        If the email is a registered user then access to group will be immediate,
        otherise an invitation will be created and emailed.

        Args:
          do_email: use legalesign to send email notification to new user

          permission:
              Permissions options:

              - 1 - administrator
              - 2 - team docs visible, create & send
              - 3 - team docs visible, send only
              - 4 - no team sent docs visible, send only
              - 5 - no team docs visible, create & send
              - 6 - team docs visible, read only

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/member/",
            body=maybe_transform(
                {
                    "email": email,
                    "group": group,
                    "do_email": do_email,
                    "permission": permission,
                },
                member_create_params.MemberCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def retrieve(
        self,
        member_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MemberResponse:
        """
        Get group member

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not member_id:
            raise ValueError(f"Expected a non-empty value for `member_id` but received {member_id!r}")
        return self._get(
            f"/member/{member_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MemberResponse,
        )

    def list(
        self,
        *,
        group: str | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MemberListResponse:
        """
        List members of groups, one user may be in one or more groups

        Args:
          group: filter list by a given group

          limit: Length of dataset to return. Use with offset query to iterate through results.

          offset: Offset from start of dataset. Use with the limit query to iterate through
              dataset.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/member/",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "group": group,
                        "limit": limit,
                        "offset": offset,
                    },
                    member_list_params.MemberListParams,
                ),
            ),
            cast_to=MemberListResponse,
        )

    def delete(
        self,
        member_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove member from group

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not member_id:
            raise ValueError(f"Expected a non-empty value for `member_id` but received {member_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/member/{member_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncMemberResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMemberResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/legalesign/legalesign-rest-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMemberResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMemberResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/legalesign/legalesign-rest-python#with_streaming_response
        """
        return AsyncMemberResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        email: str,
        group: str,
        do_email: bool | Omit = omit,
        permission: PermissionsEnum | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        If the email is a registered user then access to group will be immediate,
        otherise an invitation will be created and emailed.

        Args:
          do_email: use legalesign to send email notification to new user

          permission:
              Permissions options:

              - 1 - administrator
              - 2 - team docs visible, create & send
              - 3 - team docs visible, send only
              - 4 - no team sent docs visible, send only
              - 5 - no team docs visible, create & send
              - 6 - team docs visible, read only

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/member/",
            body=await async_maybe_transform(
                {
                    "email": email,
                    "group": group,
                    "do_email": do_email,
                    "permission": permission,
                },
                member_create_params.MemberCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def retrieve(
        self,
        member_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MemberResponse:
        """
        Get group member

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not member_id:
            raise ValueError(f"Expected a non-empty value for `member_id` but received {member_id!r}")
        return await self._get(
            f"/member/{member_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MemberResponse,
        )

    async def list(
        self,
        *,
        group: str | Omit = omit,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MemberListResponse:
        """
        List members of groups, one user may be in one or more groups

        Args:
          group: filter list by a given group

          limit: Length of dataset to return. Use with offset query to iterate through results.

          offset: Offset from start of dataset. Use with the limit query to iterate through
              dataset.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/member/",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "group": group,
                        "limit": limit,
                        "offset": offset,
                    },
                    member_list_params.MemberListParams,
                ),
            ),
            cast_to=MemberListResponse,
        )

    async def delete(
        self,
        member_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove member from group

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not member_id:
            raise ValueError(f"Expected a non-empty value for `member_id` but received {member_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/member/{member_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class MemberResourceWithRawResponse:
    def __init__(self, member: MemberResource) -> None:
        self._member = member

        self.create = to_raw_response_wrapper(
            member.create,
        )
        self.retrieve = to_raw_response_wrapper(
            member.retrieve,
        )
        self.list = to_raw_response_wrapper(
            member.list,
        )
        self.delete = to_raw_response_wrapper(
            member.delete,
        )


class AsyncMemberResourceWithRawResponse:
    def __init__(self, member: AsyncMemberResource) -> None:
        self._member = member

        self.create = async_to_raw_response_wrapper(
            member.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            member.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            member.list,
        )
        self.delete = async_to_raw_response_wrapper(
            member.delete,
        )


class MemberResourceWithStreamingResponse:
    def __init__(self, member: MemberResource) -> None:
        self._member = member

        self.create = to_streamed_response_wrapper(
            member.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            member.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            member.list,
        )
        self.delete = to_streamed_response_wrapper(
            member.delete,
        )


class AsyncMemberResourceWithStreamingResponse:
    def __init__(self, member: AsyncMemberResource) -> None:
        self._member = member

        self.create = async_to_streamed_response_wrapper(
            member.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            member.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            member.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            member.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["GroupCreateParams"]


class GroupCreateParams(TypedDict, total=False):
    name: Required[str]

    xframe_allow: bool
    """Set to true if you want to embed your signing page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .field_create_params import FieldCreateParams as FieldCreateParams
from .field_list_response import FieldListResponse as FieldListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import signer_reset_params, signer_send_reminder_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.signer_retrieve_response import SignerRetrieveResponse
from ..types.signer_retrieve_fields_response import SignerRetrieveFieldsResponse
from ..types.signer_get_rejection_reason_response import SignerGetRejectionReasonResponse

__all__ = ["SignerResource", "AsyncSignerResource"]


class SignerResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SignerResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/legalesign/legalesign-rest-python#accessing-raw-response-data-eg-headers
        """
        return SignerResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SignerResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/legalesign/legalesign-rest-python#with_streaming_response
        """
        return SignerResourceWithStreamingResponse(self)

    def retrieve(
        self,
        signer_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SignerRetrieveResponse:
        """
        Get status and details of an individual signer

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not signer_id:
            raise ValueError(f"Expected a non-empty value for `signer_id` but received {signer_id!r}")
        return self._get(
            f"/signer/{signer_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SignerRetrieveResponse,
        )

    def get_access_link(
        self,
        signer_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Returns 1-use link for signer in Location header.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not signer_id:
            raise ValueError(f"Expected a non-empty value for `signer_id` but received {signer_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            f"/signer/{signer_id}/new-link/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get_rejection_reason(
        self,
        signer_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SignerGetRejectionReasonResponse:
        """
        Returns reason signer gave for rejecting a document, if given

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not signer_id:
            raise ValueError(f"Expected a non-empty value for `signer_id` but received {signer_id!r}")
        return self._get(
            f"/signer/{signer_id}/rejection/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SignerGetRejectionReasonResponse,
        )

    def reset(
        self,
        signer_id: str,
        *,
        email: str,
        notify: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Reset to an earlier signer if forwarded

        Args:
          email: Email of signer to revert to.

          notify: Email notify current signer access is being withdrawn

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not signer_id:
            raise ValueError(f"Expected a non-empty value for `signer_id` but received {signer_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/signer/{signer_id}/reset/",
            body=maybe_transform(
                {
                    "email": email,
                    "notify": notify,
                },
                signer_reset_params.SignerResetParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def retrieve_fields(
        self,
        signer_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SignerRetrieveFieldsResponse:
        """
        Get signer form fields

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not signer_id:
            raise ValueError(f"Expected a non-empty value for `signer_id` but received {signer_id!r}")
        return self._get(
            f"/signer/{signer_id}/fields1/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SignerRetrieveFieldsResponse,
        )

    def send_reminder(
        self,
        signer_id: str,
        *,
        text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Send signer reminder email

        Args:
          text: custom message text, html will be stripped

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not signer_id:
            raise ValueError(f"Expected a non-empty value for `signer_id` but received {signer_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/signer/{signer_id}/send-reminder/",
            body=maybe_transform({"text": text}, signer_send_reminder_params.SignerSendReminderParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncSignerResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSignerResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/legalesign/legalesign-rest-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSignerResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSignerResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/legalesign/legalesign-rest-python#with_streaming_response
        """
        return AsyncSignerResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        signer_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SignerRetrieveResponse:
        """
        Get status and details of an individual signer

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not signer_id:
            raise ValueError(f"Expected a non-empty value for `signer_id` but received {signer_id!r}")
        return await self._get(
            f"/signer/{signer_id}/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SignerRetrieveResponse,
        )

    async def get_access_link(
        self,
        signer_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Returns 1-use link for signer in Location header.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not signer_id:
            raise ValueError(f"Expected a non-empty value for `signer_id` but received {signer_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            f"/signer/{signer_id}/new-link/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get_rejection_reason(
        self,
        signer_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SignerGetRejectionReasonResponse:
        """
        Returns reason signer gave for rejecting a document, if given

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not signer_id:
            raise ValueError(f"Expected a non-empty value for `signer_id` but received {signer_id!r}")
        return await self._get(
            f"/signer/{signer_id}/rejection/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SignerGetRejectionReasonResponse,
        )

    async def reset(
        self,
        signer_id: str,
        *,
        email: str,
        notify: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Reset to an earlier signer if forwarded

        Args:
          email: Email of signer to revert to.

          notify: Email notify current signer access is being withdrawn

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not signer_id:
            raise ValueError(f"Expected a non-empty value for `signer_id` but received {signer_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/signer/{signer_id}/reset/",
            body=await async_maybe_transform(
                {
                    "email": email,
                    "notify": notify,
                },
                signer_reset_params.SignerResetParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def retrieve_fields(
        self,
        signer_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SignerRetrieveFieldsResponse:
        """
        Get signer form fields

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not signer_id:
            raise ValueError(f"Expected a non-empty value for `signer_id` but received {signer_id!r}")
        return await self._get(
            f"/signer/{signer_id}/fields1/",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SignerRetrieveFieldsResponse,
        )

    async def send_reminder(
        self,
        signer_id: str,
        *,
        text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Send signer reminder email

        Args:
          text: custom message text, html will be stripped

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not signer_id:
            raise ValueError(f"Expected a non-empty value for `signer_id` but received {signer_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/signer/{signer_id}/send-reminder/",
            body=await async_maybe_transform({"text": text}, signer_send_reminder_params.SignerSendReminderParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class SignerResourceWithRawResponse:
    def __init__(self, signer: SignerResource) -> None:
        self._signer = signer

        self.retrieve = to_raw_response_wrapper(
            signer.retrieve,
        )
        self.get_access_link = to_raw_response_wrapper(
            signer.get_access_link,
        )
        self.get_rejection_reason = to_raw_response_wrapper(
            signer.get_rejection_reason,
        )
        self.reset = to_raw_response_wrapper(
            signer.reset,
        )
        self.retrieve_fields = to_raw_response_wrapper(
            signer.retrieve_fields,
        )
        self.send_reminder = to_raw_response_wrapper(
            signer.send_reminder,
        )


class AsyncSignerResourceWithRawResponse:
    def __init__(self, signer: AsyncSignerResource) -> None:
        self._signer = signer

        self.retrieve = async_to_raw_response_wrapper(
            signer.retrieve,
        )
        self.get_access_link = async_to_raw_response_wrapper(
            signer.get_access_link,
        )
        self.get_rejection_reason = async_to_raw_response_wrapper(
            signer.get_rejection_reason,
        )
        self.reset = async_to_raw_response_wrapper(
            signer.reset,
        )
        self.retrieve_fields = async_to_raw_response_wrapper(
            signer.retrieve_fields,
        )
        self.send_reminder = async_to_raw_response_wrapper(
            signer.send_reminder,
        )


class SignerResourceWithStreamingResponse:
    def __init__(self, signer: SignerResource) -> None:
        self._signer = signer

        self.retrieve = to_streamed_response_wrapper(
            signer.retrieve,
        )
        self.get_access_link = to_streamed_response_wrapper(
            signer.get_access_link,
        )
        self.get_rejection_reason = to_streamed_response_wrapper(
            signer.get_rejection_reason,
        )
        self.reset = to_streamed_response_wrapper(
            signer.reset,
        )
        self.retrieve_fields = to_streamed_response_wrapper(
            signer.retrieve_fields,
        )
        self.send_reminder = to_streamed_response_wrapper(
            signer.send_reminder,
        )


class AsyncSignerResourceWithStreamingResponse:
    def __init__(self, signer: AsyncSignerResource) -> None:
        self._signer = signer

        self.retrieve = async_to_streamed_response_wrapper(
            signer.retrieve,
        )
        self.get_access_link = async_to_streamed_response_wrapper(
            signer.get_access_link,
        )
        self.get_rejection_reason = async_to_streamed_response_wrapper(
            signer.get_rejection_reason,
        )
        self.reset = async_to_streamed_response_wrapper(
            signer.reset,
        )
        self.retrieve_fields = async_to_streamed_response_wrapper(
            signer.retrieve_fields,
        )
        self.send_reminder = async_to_streamed_response_wrapper(
            signer.send_reminder,
        )
